@@ -408,11 +408,17 @@ int raw_socket_read (CONNECTION* conn, char* buf, size_t len)
 {
   int rc;
 
-  if ((rc = read (conn->fd, buf, len)) == -1)
+  do
+  {
+    rc = read (conn->fd, buf, len);
+  } while (rc < 0 && errno == EINTR);
+
+  if (rc < 0)
   {
     mutt_error (_("Error talking to %s (%s)"), conn->account.host,
 		strerror (errno));
     mutt_sleep (2);
+    return -1;
   }
 
   return rc;
@@ -421,15 +427,27 @@ int raw_socket_read (CONNECTION* conn, char* buf, size_t len)
 int raw_socket_write (CONNECTION* conn, const char* buf, size_t count)
 {
   int rc;
+  size_t sent = 0;
 
-  if ((rc = write (conn->fd, buf, count)) == -1)
+  do
   {
-    mutt_error (_("Error talking to %s (%s)"), conn->account.host,
-		strerror (errno));
-    mutt_sleep (2);
-  }
+    do
+    {
+      rc = write (conn->fd, buf + sent, count - sent);
+    } while (rc < 0 && errno == EINTR);
 
-  return rc;
+    if (rc < 0)
+    {
+      mutt_error (_("Error talking to %s (%s)"), conn->account.host,
+                  strerror (errno));
+      mutt_sleep (2);
+      return -1;
+    }
+
+    sent += rc;
+  } while (sent < count);
+
+  return sent;
 }
 
 int raw_socket_poll (CONNECTION* conn, time_t wait_secs)
@@ -499,7 +517,7 @@ int raw_socket_open (CONNECTION* conn)
 
   snprintf (port, sizeof (port), "%d", conn->account.port);
   
-# ifdef HAVE_LIBIDN
+# if defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2)
   if (idna_to_ascii_lz (conn->account.host, &host_idna, 1) != IDNA_SUCCESS)
   {
     mutt_error (_("Bad IDN \"%s\"."), conn->account.host);
@@ -514,7 +532,7 @@ int raw_socket_open (CONNECTION* conn)
   
   rc = getaddrinfo (host_idna, port, &hints, &res);
 
-# ifdef HAVE_LIBIDN
+# if defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2)
   FREE (&host_idna);
 # endif
 
@@ -558,7 +576,7 @@ int raw_socket_open (CONNECTION* conn)
   sin.sin_port = htons (conn->account.port);
   sin.sin_family = AF_INET;
 
-# ifdef HAVE_LIBIDN
+# if defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2)
   if (idna_to_ascii_lz (conn->account.host, &host_idna, 1) != IDNA_SUCCESS)
   {
     mutt_error (_("Bad IDN \"%s\"."), conn->account.host);
@@ -573,7 +591,7 @@ int raw_socket_open (CONNECTION* conn)
 
   he = gethostbyname (host_idna);
 
-# ifdef HAVE_LIBIDN
+# if defined(HAVE_LIBIDN) || defined(HAVE_LIBIDN2)
     FREE (&host_idna);
 # endif
 
